"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFormatterFromArgs = exports.getCellRenderer = exports.getFormatter = exports.defaultDatetimeFormatter = exports.getDatetimeFormatter = exports.getFloatFormatter = exports.booleanFormatter = exports.boolDisplayer = exports.objFormatter = exports.getObjectFormatter = exports.dateDisplayerDefault = exports.isValidDate = exports.getStringFormatter = exports.basicIntFormatter = void 0;
const DFWhole_1 = require("./DFWhole");
const lodash_1 = __importDefault(require("lodash"));
const HistogramCell_1 = require("./HistogramCell");
/*
  this code should all be unit tested and in examples. Examples will
  show potential developers how this behaves. Examples should be made
  inside of AG-Grid, and independently.
  */
exports.basicIntFormatter = new Intl.NumberFormat('en-US', {
    minimumFractionDigits: 0,
    maximumFractionDigits: 3,
});
const getStringFormatter = (args) => {
    const stringFormatter = (params) => {
        const val = params.value;
        if (val && args.max_length) {
            return val.slice(0, args.max_length);
        }
        return val;
    };
    return stringFormatter;
};
exports.getStringFormatter = getStringFormatter;
const dictDisplayer = (val) => {
    const objBody = lodash_1.default.map(val, (value, key) => `'${key}': ${objDisplayer(value)}`).join(',');
    return `{ ${objBody} }`;
};
const isValidDate = (possibleDate) => {
    if (lodash_1.default.isDate(possibleDate) && isFinite(possibleDate.getTime())) {
        return true;
    }
    return false;
};
exports.isValidDate = isValidDate;
const DEFAULT_DATE_FORMAT = {
    year: 'numeric',
    month: 'numeric',
    day: 'numeric',
    hour: 'numeric',
    minute: 'numeric',
    second: 'numeric',
    hour12: false,
};
const dateDisplayerDefault = (d) => {
    const fullStr = d.toLocaleDateString('en-CA', DEFAULT_DATE_FORMAT);
    const [dateStr, timeStr] = fullStr.split(',');
    const retVal = `${dateStr} ${timeStr}`;
    return retVal;
};
exports.dateDisplayerDefault = dateDisplayerDefault;
const objDisplayer = (val) => {
    if (val === undefined || val === null) {
        return 'None';
    }
    else if (lodash_1.default.isArray(val)) {
        return `[ ${val.map(objDisplayer).join(', ')}]`;
    }
    else if (lodash_1.default.isBoolean(val)) {
        return (0, exports.boolDisplayer)(val);
    }
    else if (lodash_1.default.isObject(val)) {
        return dictDisplayer(val);
    }
    else {
        return val.toString();
    }
    return val;
};
const getObjectFormatter = (fArgs) => {
    const objFormatter = (params) => {
        const val = params.value;
        const fullString = objDisplayer(val);
        if (fArgs.max_length) {
            return fullString.slice(0, fArgs.max_length);
        }
        else {
            return fullString;
        }
    };
    return objFormatter;
};
exports.getObjectFormatter = getObjectFormatter;
exports.objFormatter = (0, exports.getObjectFormatter)({ displayer: 'obj' });
const boolDisplayer = (val) => {
    if (val === true) {
        return 'True';
    }
    else if (val === false) {
        return 'False';
    }
    return '';
};
exports.boolDisplayer = boolDisplayer;
const booleanFormatter = (params) => {
    const val = params.value;
    return (0, exports.boolDisplayer)(val);
};
exports.booleanFormatter = booleanFormatter;
const getIntegerFormatter = (hint) => {
    const commas = Math.floor(hint.max_digits / 3);
    const totalWidth = commas + hint.max_digits;
    const formatter = new Intl.NumberFormat('en-US');
    const numericFormatter = (params) => {
        const val = params.value;
        if (val === null) {
            return '';
        }
        return formatter.format(params.value).padStart(totalWidth, ' ');
    };
    return numericFormatter;
};
const getFloatFormatter = (hint) => {
    const floatFormatter = new Intl.NumberFormat('en-US', {
        minimumFractionDigits: hint.min_fraction_digits,
        maximumFractionDigits: hint.max_fraction_digits,
    });
    return (params) => {
        if (params.value === null) {
            return '';
        }
        const res = floatFormatter.format(params.value);
        if (!lodash_1.default.includes(res, '.')) {
            const padLength = res.length + hint.max_fraction_digits + 1;
            return res.padEnd(padLength);
        }
        else {
            const fracPart = res.split('.')[1];
            const padLength = hint.max_fraction_digits - fracPart.length + res.length;
            return res.padEnd(padLength);
        }
    };
};
exports.getFloatFormatter = getFloatFormatter;
const getDatetimeFormatter = (colHint) => {
    return (params) => {
        const val = params.value;
        if (val === null || val === undefined) {
            return '';
        }
        const d = new Date(val);
        if (!(0, exports.isValidDate)(d)) {
            return '';
        }
        return d.toLocaleDateString(colHint.locale, colHint.args);
    };
};
exports.getDatetimeFormatter = getDatetimeFormatter;
const defaultDatetimeFormatter = (params) => {
    const val = params.value;
    if (val === null || val === undefined) {
        return '';
    }
    const d = new Date(val);
    if (!(0, exports.isValidDate)(d)) {
        return '';
    }
    return (0, exports.dateDisplayerDefault)(d);
};
exports.defaultDatetimeFormatter = defaultDatetimeFormatter;
function getFormatter(fArgs) {
    switch (fArgs.displayer) {
        case 'integer':
            return getIntegerFormatter(fArgs);
        case 'string':
            return (0, exports.getStringFormatter)(fArgs);
        case 'datetimeDefault':
            return exports.defaultDatetimeFormatter;
        case 'datetimeLocaleString':
            return (0, exports.getDatetimeFormatter)(fArgs);
        case 'float':
            return (0, exports.getFloatFormatter)(fArgs);
        case 'boolean':
            return exports.booleanFormatter;
        case 'obj':
            return (0, exports.getObjectFormatter)(fArgs);
        default:
            return (0, exports.getStringFormatter)({ displayer: 'string' });
    }
}
exports.getFormatter = getFormatter;
/*
{
  cellRenderer: 'agCheckboxCellRenderer',
  cellRendererParams: {   disabled: true}
  }
  */
function getCellRenderer(crArgs) {
    switch (crArgs.displayer) {
        case 'histogram':
            return HistogramCell_1.HistogramCell;
        case 'linkify':
            return HistogramCell_1.LinkCellRenderer;
        case 'Base64PNGImageDisplayer':
            return HistogramCell_1.Base64PNGDisplayer;
        case 'boolean_checkbox':
            return 'agCheckboxCellRenderer';
        case 'SVGDisplayer':
            return HistogramCell_1.SVGDisplayer;
    }
}
exports.getCellRenderer = getCellRenderer;
function getFormatterFromArgs(dispArgs) {
    if (lodash_1.default.includes(DFWhole_1.cellRendererDisplayers, dispArgs.displayer)) {
        return undefined;
    }
    const fArgs = dispArgs;
    return getFormatter(fArgs);
}
exports.getFormatterFromArgs = getFormatterFromArgs;
//# sourceMappingURL=Displayer.js.map